/***********************************************************************/
/*  This file is part of the ARM Toolchain package                     */
/*  Copyright (c) 2010 Keil - An ARM Company. All rights reserved.     */
/***********************************************************************/
/*                                                                     */
/*  FlashDev.C:  Flash Programming Functions adapted                   */
/*               for K1921VK035 64kB Flash                            */
/*                                                                     */
/***********************************************************************/

#include "..\FlashOS.H"        // FlashOS Structures
#include <stdint.h>

#define M8(adr)  (*((volatile unsigned char  *) (adr)))
#define M16(adr) (*((volatile unsigned short *) (adr)))
#define M32(adr) (*((volatile unsigned long  *) (adr)))
	
#define MFLASH_MAGIC_KEY		0xC0DE0000
#define MFLASH_READ					0xC0DE0001
#define MFLASH_WRITE				0xC0DE0002
#define MFLASH_SECTOR_ERASE	0xC0DE0004
#define MFLASH_FULL_ERASE		0xC0DE0008
#define MFLASH_NVR					0x00000100

#define   __I     volatile const       /*!< Defines 'read only' permissions                 */
#define     __O     volatile             /*!< Defines 'write only' permissions                */
#define     __IO    volatile             /*!< Defines 'read / write' permissions              */

/* ================================================================================ */
/* ================                  MFLASH                  ================ */
/* ================================================================================ */

typedef struct {
    __IO uint32_t ADDR;                                              /*!< ADDR    : type used for word access */
    __IO uint32_t DATA[2];
    __IO uint32_t Reserved0[6];
/*!< Command Register */
    __IO uint32_t CMD;                                               /*!< CMD    : type used for word access */
																	/*!< Status Register */
    __IO uint32_t STAT;                                              /*!< STAT    : type used for word access */
                                                             /*!< Control Register */
    __IO uint32_t CTRL;                                              /*!< CTRL    : type used for word access */
                                                              /*!< Cache Test Register */
    __O uint32_t CTEST;                                              /*!< CTEST    : type used for word access */
                                                               /*!< ICACHE Status Register */
    __I uint32_t ICSTAT;                                             /*!< ICSTAT    : type used for word access */
                                                              /*!< DCACHE Status Register */
    __I uint32_t DCSTAT;                                             /*!< DCSTAT    : type used for word access */

    __IO uint32_t Reserved1[5];
                                                             /*!< Read hold time register */
    __IO uint32_t TRH;                                               /*!< TRH    : type used for word access */
                                                              /*!< Time to WEn register */
    __IO uint32_t TNVS;                                              /*!< TNVS    : type used for word access */
                                                               /*!< Time from Wen low to PROG2 high register */
    __IO uint32_t TPGS;                                              /*!< TPGS    : type used for word access */
                                                             /*!< Program byte time register */
    __IO uint32_t TPROG;                                             /*!< TPROG    : type used for word access */
                                                              /*!< Address/data setup/hold time register */
    __IO uint32_t TADHS;                                             /*!< TADHS    : type used for word access */
                                                              /*!< PROG 2 Low to WEb high Hold time register */
    __IO uint32_t TPGH;                                              /*!< TPGH    : type used for word access */
                                                              /*!< Recovery time after write register */
    __IO uint32_t TRW;                                               /*!< TRW    : type used for word access */
                                                              /*!< Recovery time after erase register */
    __IO uint32_t TRE;                                               /*!< TRE    : type used for word access */
                                                              /*!< Erase sector time register */
    __IO uint32_t TES;                                               /*!< TES    : type used for word access */
                                                              /*!< Erase all flash time register */
    __IO uint32_t TEA;                                               /*!< TEA    : type used for word access */
                                                             /*!< Boot Mode Disable register */
    __IO uint32_t BDIS;                                              /*!< BDIS    : type used for word access */
} MFLASH_TypeDef;


#define MFLASH_BASE             (0x40030000UL)
#define MFLASH               	  ((MFLASH_TypeDef    *) MFLASH_BASE)

#define FULL_FLASH_ERASE_ADDR	0x40040014
// Define for Read SERVICE MODE
#define SERV_DETECT ((M32(FULL_FLASH_ERASE_ADDR) >> 31)&0x01)

/* 
   Mandatory Flash Programming Functions (Called by FlashOS):
                int Init        (unsigned long adr,   // Initialize Flash
                                 unsigned long clk,
                                 unsigned long fnc);
                int UnInit      (unsigned long fnc);  // De-initialize Flash
                int EraseSector (unsigned long adr);  // Erase Sector Function
                int ProgramPage (unsigned long adr,   // Program Page Function
                                 unsigned long sz,
                                 unsigned char *buf);

   Optional  Flash Programming Functions (Called by FlashOS):
                int BlankCheck  (unsigned long adr,   // Blank Check
                                 unsigned long sz,
                                 unsigned char pat);
                int EraseChip   (void);               // Erase complete Device
      unsigned long Verify      (unsigned long adr,   // Verify Function
                                 unsigned long sz,
                                 unsigned char *buf);

       - BlanckCheck  is necessary if Flash space is not mapped into CPU memory space
       - Verify       is necessary if Flash space is not mapped into CPU memory space
       - if EraseChip is not provided than EraseSector for all sectors is called
*/


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

  /* Add your Code */
	/* Configurate time acces to FLASH */

  return (0);                                  // Finished without Errors
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  /* Add your Code */
	/* Configurate time acces to FLASH defualt value on 100MHz*/

  return (0);                                  // Finished without Errors
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {
  /* Add your Code */
  if (SERV_DETECT) M32(FULL_FLASH_ERASE_ADDR)=1;
  else {
	MFLASH->CMD = MFLASH_FULL_ERASE;
	__nop();
	__nop();
	__nop();
	__nop();	
	while (MFLASH->STAT & 1);
  }	
  return (0);                                  // Finished without Errors
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {
  /* Add your Code */
	MFLASH->ADDR = adr;
	MFLASH->CMD = MFLASH_SECTOR_ERASE;
	__nop();
	__nop();
	__nop();
	__nop();	
	while (MFLASH->STAT & 1);
  return (0);                                  // Finished without Errors
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {

  /* Add your Code */
           int  i;
                           // Number of 2*32-byte Buffers
  for (i = 0; i < ((sz + 7) / 8); i++)  {
		MFLASH->ADDR = adr;
		MFLASH->DATA[0] = *((unsigned long *) buf); 	// Write Data to Buffers
		buf += 4;
		MFLASH->DATA[1] = *((unsigned long *) buf); 	// Write Data to Buffers
		buf += 4;                                 			// Next Data
		adr += 8;                                 			// Next Address
		MFLASH->CMD = MFLASH_WRITE; // WRITE DATA to BOOTFLASH
	__nop();
	__nop();
	__nop();
	__nop();		
		while (MFLASH->STAT & 1);
	}
  return (0);                                  // Finished without Errors
}
